==> 5.3.0.tar.gz: lib/src/domain/base_state.dart <==
import 'dart:async';

import 'package:equatable/equatable.dart';
import 'package:osam/osam.dart';
import 'package:osam/src/util/value_stream.dart';

typedef ValueMapper<ST, V> = V Function(ST state);
typedef StateMapper<ST> = void Function(ST state);

// ignore: must_be_immutable
abstract class BaseState<ST> extends Equatable {
  StreamController _stateBroadcaster;

  BaseState() {
    _stateBroadcaster =
        StreamController<ST>.broadcast(onListen: () => _isObserved = true, onCancel: () => _isObserved = false);
  }

  int _lastKnownHashCode = -1;
  bool _isObserved = false;

  @override
  List<Object> get props;

  ValueStream<ST> get _stateStream => ValueStream(_stateBroadcaster.stream.map((state) => state), this as ST);

  ValueStream<V> propertyStream<V>(ValueMapper<ST, V> mapper) => ValueStream(
      (PropertyStream<V>(
              _stateStream.stream.map<V>((state) {
                try {
                  return mapper(state);
                } catch (e) {
                  handleError(e);
                  return null;
                }
              }).where((property) => property != null),
              mapper(_stateStream.value)))
          .propertyStream,
      mapper(_stateStream.value));

  // could be awaited to refresh every synced changes
  void update(StateMapper<ST> change) async {
    try {
      change(this as ST);
    } catch (e) {
      handleError(e);
    }
    if (_lastKnownHashCode != hashCode && _isObserved) {
      await _stateBroadcaster.add(this as ST);
      _lastKnownHashCode = hashCode;
    }
  }

  void handleError(Object error) => print(error);

  void destruct() async => await _stateBroadcaster.close();

  void throwError(Object error) async => await _stateBroadcaster.addError(error);

  void clear() {}
}

class PropertyStream<V> {
  final Stream<V> _inputStream;
  final V _firstValue;
  int _valueHashCode = -1;

  PropertyStream(this._inputStream, this._firstValue) {
    _valueHashCode = _ComparableWrapper(_firstValue).hashCode;
  }

  Stream<V> get propertyStream => _inputStream.map((next) {
        final comparableWrapper = _ComparableWrapper(next);
        final result = _valueHashCode == comparableWrapper.hashCode;
        _valueHashCode = comparableWrapper.hashCode;
        return result ? null : next;
      }).where((event) => event != null);
}

class _ComparableWrapper<V> extends Equatable {
  final V _value;

  _ComparableWrapper(this._value);

  @override
  List<Object> get props => [_value];
}

==> 5.3.2.tar.gz: lib/src/domain/base_state.dart <==
import 'dart:async';

import 'package:equatable/equatable.dart';
import 'package:osam/osam.dart';
import 'package:osam/src/util/value_stream.dart';

typedef ValueMapper<ST, V> = V Function(ST state);
typedef StateMapper<ST> = void Function(ST state);

// ignore: must_be_immutable
abstract class BaseState<ST> extends Equatable {
  StreamController _stateBroadcaster;

  BaseState() {
    _stateBroadcaster =
        StreamController<ST>.broadcast(onListen: () => _isObserved = true, onCancel: () => _isObserved = false);
  }

  int _lastKnownHashCode = -1;
  bool _isObserved = false;

  @override
  List<Object> get props;

  ValueStream<ST> get _stateStream => ValueStream(_stateBroadcaster.stream.map((state) => state), this as ST);

  ValueStream<V> propertyStream<V>(ValueMapper<ST, V> mapper) => ValueStream(
      (PropertyStream<V>(
              _stateStream.stream.map<V>((state) {
                try {
                  return mapper(state);
                } catch (e) {
                  handleError(e);
                  return null;
                }
              }).where((property) => property != null),
              mapper(_stateStream.value)))
          .propertyStream,
      mapper(_stateStream.value));

  void update(StateMapper<ST> change) {
    try {
      change(this as ST);
    } catch (e) {
      handleError(e);
    }
    if (_lastKnownHashCode != hashCode && _isObserved) {
      _stateBroadcaster.sink.add(this as ST);
      _lastKnownHashCode = hashCode;
    }
  }

  void handleError(Object error) => print(error);

  void destruct() async => await _stateBroadcaster.close();

  void throwError(Object error) async => await _stateBroadcaster.addError(error);

  void clear() {}
}

class PropertyStream<V> {
  final Stream<V> _inputStream;
  final V _firstValue;
  int _valueHashCode = -1;

  PropertyStream(this._inputStream, this._firstValue) {
    _valueHashCode = _ComparableWrapper(_firstValue).hashCode;
  }

  Stream<V> get propertyStream => _inputStream.map((next) {
        final comparableWrapper = _ComparableWrapper(next);
        final result = _valueHashCode == comparableWrapper.hashCode;
        _valueHashCode = comparableWrapper.hashCode;
        return result ? null : next;
      }).where((event) => event != null);
}

class _ComparableWrapper<V> extends Equatable {
  final V _value;

  _ComparableWrapper(this._value);

  @override
  List<Object> get props => [_value];
}
